@@ -21,20 +21,22 @@
 #' \donttest{
 #' ## load iris data
 #' data(iris)
-#' X     <- as.matrix(iris[,1:4])
-#' label <- as.integer(iris$Species)
+#' set.seed(100)
+#' subid = sample(1:150,50)
+#' X     = as.matrix(iris[subid,1:4])
+#' lab   = as.factor(iris[subid,5])
 #'
 #' ## let's compare with other methods
 #' out1 <- do.pca(X, ndim=2)
-#' out2 <- do.sne(X, ndim=2)
+#' out2 <- do.lda(X, ndim=2, label=lab)
 #' out3 <- do.idmap(X, ndim=2, engine="NNP")
 #'
 #' ## visualize
 #' opar <- par(no.readonly=TRUE)
 #' par(mfrow=c(1,3))
-#' plot(out1$Y, col=label, main="PCA")
-#' plot(out2$Y, col=label, main="SNE")
-#' plot(out3$Y, col=label, main="IDMAP")
+#' plot(out1$Y, pch=19, col=lab, main="PCA")
+#' plot(out2$Y, pch=19, col=lab, main="LDA")
+#' plot(out3$Y, pch=19, col=lab, main="IDMAP")
 #' par(opar)
 #' }
 #'
@@ -43,7 +45,7 @@
 #'
 #' @seealso \code{\link{do.nnp}}, \code{\link{do.fastmap}}
 #' @rdname nonlinear_IDMAP
-#' @concept nonlinear_methods 
+#' @concept nonlinear_methods
 #' @export
 do.idmap <- function(X, ndim=2, preprocess=c("null","center","scale","cscale","whiten","decorrelate"), engine=c("NNP","FastMap")){
   ########################################################################
@@ -81,7 +83,11 @@ do.idmap <- function(X, ndim=2, preprocess=c("null","center","scale","cscale","w
 
   ########################################################################
   ## 4. typechecking
-  engine = match.arg(engine)
+  if (missing(engine)){
+    engine = "NNP"
+  } else {
+    engine = match.arg(engine)
+  }
   if (engine=="FastMap"){
     k=ndim
     Dold=D
