@@ -24,6 +24,7 @@
 #' }
 #'
 #' @examples
+#' \donttest{
 #' ## load iris data
 #' data(iris)
 #' X     = as.matrix(iris[,1:4])
@@ -37,17 +38,18 @@
 #' ## Visualize
 #' opar <- par(no.readonly=TRUE)
 #' par(mfrow=c(1,3))
-#' plot(outM$Y, col=label, main="MDS")
-#' plot(out1$Y, col=label, main="SPE with L2 norm")
-#' plot(out2$Y, col=label, main="SPE with L1 norm")
+#' plot(outM$Y, pch=19, col=label, main="MDS")
+#' plot(out1$Y, pch=19, col=label, main="SPE with L2 norm")
+#' plot(out2$Y, pch=19, col=label, main="SPE with L1 norm")
 #' par(opar)
+#' }
 #'
 #' @references
 #' \insertRef{agrafiotis_stochastic_2003}{Rdimtools}
 #'
 #' @author Kisung You
 #' @rdname nonlinear_SPE
-#' @concept nonlinear_methods 
+#' @concept nonlinear_methods
 #' @export
 do.spe <- function(X, ndim=2, proximity=function(x){dist(x, method="euclidean")},
                    C = 50, S = 50, lambda = 1, drate=0.9){
@@ -83,7 +85,7 @@ do.spe <- function(X, ndim=2, proximity=function(x){dist(x, method="euclidean")}
   ## MAIN COMPUTATION
   # 1. Proximity Matrix R : don't forget to make it into a matrix.
   R = proximity(X)
-  if ((inherits(R, "dist"))||(class(R)=="dist")){
+  if (inherits(R, "dist")){
     R = as.matrix(R)
   }
   # 2. initial coordinates
